/*
 * Mage.h
 *
 * Copyright 2000, Swarthmore College
 */

#ifdef __cplusplus
extern "C" {
#endif
  
/* the number of entries in the state */
#define NUM_STATE        46

/* the number of sonars and infrareds */
#define SONARS           16
#define INFRAREDS        16

/*
 * The following defines allow you to access the State vector in a 
 * more readable way.
 */
#define STATE_IR_0                 1
#define STATE_IR_1                 2
#define STATE_IR_2                 3
#define STATE_IR_3                 4
#define STATE_IR_4                 5
#define STATE_IR_5                 6
#define STATE_IR_6                 7
#define STATE_IR_7                 8
#define STATE_IR_8                 9
#define STATE_IR_9                 10
#define STATE_IR_10                11
#define STATE_IR_11                12
#define STATE_IR_12                13
#define STATE_IR_13                14
#define STATE_IR_14                15
#define STATE_IR_15                16

#define STATE_SONAR_0              17
#define STATE_SONAR_1              18
#define STATE_SONAR_2              19
#define STATE_SONAR_3              20
#define STATE_SONAR_4              21
#define STATE_SONAR_5              22
#define STATE_SONAR_6              23
#define STATE_SONAR_7              24
#define STATE_SONAR_8              25
#define STATE_SONAR_9              26
#define STATE_SONAR_10             27
#define STATE_SONAR_11             28
#define STATE_SONAR_12             29
#define STATE_SONAR_13             30
#define STATE_SONAR_14             31
#define STATE_SONAR_15             32
 
#define STATE_BUMPER               33

#define STATE_X                    34
#define STATE_Y                    35
#define STATE_T                    36

#define STATE_VEL_TRANS            37
#define STATE_VEL_STEER            38
#define STATE_ACC_TRANS            39
#define STATE_ACC_STEER            40
#define STATE_TRANS                41  

#define STATE_JUMPX                42
#define STATE_JUMPY                43
#define STATE_JUMPT                44
#define STATE_JUMP                 45

/********************
 *                  *
 * Global Variables *
 *                  *
 ********************/

/* The State vector is an array of 45 long integers:
 *
 *   State[1 ... 16] - stores the 16 infrared data.
 *   State[17 ... 32] - stores the 16 sonar data.
 *   State[33] - stores bumper data
 *   State[34 ... 36] - stores robot configuration
 *   State[37 ... 40] - stores robot's current velocities and accelerations
 *   State[41] - motor status: the lowest bit is set to 1 if the translational
 *               motor is active; the second lowest bit is set to 1 if the 
 *               steering motor is active; the third lowest bit is set to 1 if 
 *               the turret motor is active.
 */

  // extern long State[NUM_STATE];

/*****************************
 *                           *
 * Robot Interface Functions *
 *                           *
 *****************************/

void connectRobot(long* St);

int disconnectRobot();

/*
 * vm - velocity mode, command the robot to move at translational
 *      velocity = tv and steering velocity = sv.  The robot will 
 *      continue to move at these velocities until
 *      either it receives another command or this command has been
 *      timed out (in which case it will stop its motion).
 *
 * parameters: 
 *    int t_vel, s_vel  -- the desired translation and steering velocities.
 *                       t_vm is in mm/sec, s_vm is in radians/1000/sec.
 *          NOTE -- vm uses a default acceleration.
 */
int vm(long t_vel, long s_vel);
  
/*
 * vam - velocity acceleration mode.  same as vm, but with accelerations, too
 *
 * parameters: 
 *    int t_vel, s_vel  -- the desired translation and steering velocities.
 *                       t_vm is in mm/sec, s_vm is in radians/1000/sec.
 *    int t_acc, s_acc
 */
int vam(long t_vel, long t_acc, long s_vel, long s_acc);

/*
 * zr - zeroing the robot, align steering and turret with bumper zero.
 *      The position, steering and turret angles are all set to zero.
 *      This function returns when the zeroing process has completed.
 */
int zr(void);
int jump(long x, long y, long t);

/*  Activate and deactivate the infrared sensor array */
int irOn(void);
int irOff(void);

/*  Activate and deactivate the sonar sensor array */
int sonarOn(void);
int sonarOff(void);

