// NOTE:
// This code relies heavily on the Canon VC-C4 command set.  
//
//  NAPPOC (Not A Polished Piece Of Code)
//
//  Nathaniel Fairfield [than@sccs.swarthmore.edu]

#include <sys/ioctl.h>
#include <sys/types.h>
#include <math.h>

#include <unistd.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <string.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>
#include <sys/timeb.h>
#include <pthread.h>
#include "serial.h"
#include "ptzClient.h"

unsigned char cmdbuf[32];
unsigned char newbuf[32];
int cmdLength;
int fd;

#define MIN(x, y) (( (x) < (y) ) ? (x) : (y))
#define MAX(x, y) (( (x) > (y) ) ? (x) : (y))


// globals
short pan;
short tilt;
short panout;
short tiltout;
short zoom;
short ptzStale = 1;
short trackState = 0; // 0 setting, 1 tracking, 2 lost

/***********************
    Utility functions 
 ***********************/

int readyPTZ () 
{
  if (cmdLength == 0) return 1;
  else return 0;
}

int donePTZ ()
{
  // fprintf(stderr,"Panning-- pan: %d, panout: %d, tilt: %d, tiltout: %d\n",pan,panout,tilt,tiltout); 
  if ((pan == panout) && (tilt == tiltout)) return 1;
  else return 0;
}

void hexPrint(unsigned char buf[], unsigned char trm)
{
  int ii = 0;
  fprintf(stderr,"hexPrint:  ");
  while (buf[ii] != trm) fprintf(stderr," %02X",buf[ii++]);
  fprintf(stderr," %02X",buf[ii]);
  fprintf(stderr,"\n");
}

/***********************
  PTZ Command functions
 ***********************/

void homePTZ()
{
  cmdLength = 6;
  sprintf(cmdbuf,"%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0x57,0xEF);
}

void setZoom(int zoom)
{
}

void setPanTilt(short panin, short tiltin)
{
  short tpan, ttilt;
  panin = 6282 - panin;
  tiltin-=1571;
  if (panin > 3141) panin-=6282;
  if (tiltin > 3141) tiltin-=6282;
  pan = (int) (panin>>1); // / 1571.0 * 800);
  tilt = (int) (tiltin>>1); // / 1571.0 * 800);

  tpan = pan;
  ttilt = tilt;

  tpan = MIN(890,tpan);
  tpan = MAX(-890,tpan);
  ttilt = MIN(800,ttilt);
  ttilt = MAX(-267,ttilt);
  tpan = 32768 + tpan;
  ttilt = 32768 + ttilt;
  cmdLength = 14;
  sprintf(cmdbuf,"%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
	  0xFF,0x30,0x30,0x00,0x62,
	  ((tpan >> 12) & 0x0F)+((((tpan >> 12) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((tpan >> 8 ) & 0x0F)+((((tpan >> 8 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((tpan >> 4 ) & 0x0F)+((((tpan >> 4 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((tpan >> 0 ) & 0x0F)+((((tpan >> 0 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 12) & 0x0F)+((((ttilt >> 12) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 8 ) & 0x0F)+((((ttilt >> 8 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 4 ) & 0x0F)+((((ttilt >> 4 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  ((ttilt >> 0 ) & 0x0F)+((((ttilt >> 0 ) & 0x0F) > 0x09) ? (7) :(0)) + 0x30,
	  0xEF
	  );

}

void initPTZ ()
{
/* Switch to serial control mode */
  cmdLength = 7;
  sprintf(cmdbuf,"%c%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0x90,0x30,0xEF);
  while(!readyPTZ());
/* Turn off auto-exposure */
  cmdLength = 7;
  sprintf(cmdbuf,"%c%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0xA5,0x33,0xEF);
  while(!readyPTZ());

 /*
  cmdLength = 7;
   sprintf(cmdbuf,"%c%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0xA7,0x31,0xEF);
   while(!readyPTZ());
  */
}

void getPTZ (int pt[3])
{
  pt[0] = (int) panout<<1;
  pt[1] = (int) tiltout<<1;
}

void *watchPTZ (void *arg)
{
  int cmdLengthTemp;
  unsigned char tmpbuf[128];
  unsigned char msgbuf[128];
  while (1) 
    {
      // fprintf(stderr,".");
      usleep(100000);
      
      if (cmdLength > 0) 
	{
	  // fprintf(stderr,"Sending ");
	  // hexPrint(cmdbuf,0xEF);
	  write(fd,cmdbuf,cmdLength);
	  cmdLengthTemp = cmdLength;
	  cmdLength = 0;
	} else {
	  // if no command, get cameraposition...
	  sprintf(tmpbuf,"%c%c%c%c%c%c",0xFF,0x30,0x30,0x00,0x63,0xEF);
	  write(fd,tmpbuf,6);
	}
      while (1) 
	{ 
	  usleep(40000);
	  
	  sprintf(msgbuf," ");
	  read(fd,msgbuf,255);
	  
	  if (msgbuf[0] == 0xFE) 
	    {
	      // fprintf(stderr,"Got ");
	      // hexPrint(msgbuf,0xEF);
	      
	      if ((msgbuf[1] == 0x30)
		  && (msgbuf[2] == 0x30)
		  && (msgbuf[3] == 0x30)
		  && (msgbuf[4] == 0x30))
		{
		  if (msgbuf[5] != 0xEF) {
		    
		    panout = (((msgbuf[5]&0x0F)+((msgbuf[5]>0x40)?(0x09):(0)))<<8)
		      + (((msgbuf[6]&0x0F)+((msgbuf[6]>0x40)?(0x09):(0)))<<8) 
		      + (((msgbuf[7]&0x0F)+((msgbuf[7]>0x40)?(0x09):(0)))<<4) 
		      + ((msgbuf[8]&0x0F)+((msgbuf[8]>0x40)?(0x09):(0)));
		    tiltout = (((msgbuf[9]&0x0F)+((msgbuf[9]>0x40)?(0x09):(0)))<<8)
		      + (((msgbuf[10]&0x0F)+((msgbuf[10]>0x40)?(0x09):(0)))<<8) 
		      + (((msgbuf[11]&0x0F)+((msgbuf[11]>0x40)?(0x09):(0)))<<4) 
		      + ((msgbuf[12]&0x0F)+((msgbuf[12]>0x40)?(0x09):(0)));
		    panout -= 2048;
		    tiltout -= 2048;
		    
		    if (panout > 2048) panout-=3840;
		    if (tiltout > 2048) tiltout-=3840;
		  }
		}
	      else // error!!!
		{
		  fprintf(stderr,"PTZ error:  resending command.  Error ");
		  hexPrint(msgbuf,0xEF);
		  if (cmdLength == 0) cmdLength = cmdLengthTemp;
		  break;
		}
	      break;
	    }
	}
    } 
}

int startPTZ()
{
  pthread_t *watcher;
  cmdLength = 0;
  cmdbuf[0] = 0x01;
  
  fd = openSerialPort("/dev/ttyS3", 9600);
  
  watcher = malloc(sizeof(pthread_t)); 
  pthread_create(watcher,NULL,&watchPTZ,NULL);
  
  while (!readyPTZ());
  initPTZ();
  while (!readyPTZ());

  return 0;
}













