// NOTE:
// This code relies heavily on the Sony DVI-D30/D31 command set.  The full
// list of commands can be found at:
// http://www.world.sony.com/Electronics/ISP/dtp/PDF/commandlist/D30E99BCL.pdf
//
//  NAPPOC (Not A Polished Piece Of Code)
//
//  Nathaniel Fairfield [than@sccs.swarthmore.edu]

#include <sys/ioctl.h>
#include <sys/types.h>
#include <math.h>

#include <unistd.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <string.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>
#include <sys/timeb.h>
#include <pthread.h>
#include "serial.h"
#include "ptzClient.h"

unsigned char cmdbuf[32];
int fd;

// globals
short pan;
short tilt;
short zoom;
short ptzStale = 1;
short trackState=0; // 0 setting, 1 tracking, 2 lost

/***********************
    Utility functions 
 ***********************/

int readyPTZ () {
  if (cmdbuf[0]==0) return 1;
  else return 0;
}

int findCmdLength(){
  int i;
  i = 0;
  while (cmdbuf[i]!=0xFF) i++;
  return i+1;
}

/***********************
  PTZ Command functions
 ***********************/

int homePTZ()
{
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x06;
  cmdbuf[3] = 0x04;
  cmdbuf[4] = 0xFF;
  return 0;
}

int setZoom(int zoom)
{
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x04;
  cmdbuf[3] = 0x47;
  cmdbuf[4] = (zoom >> 12);
  cmdbuf[5] = (zoom >> 8) & 0x00F;
  cmdbuf[6] = (zoom >> 4) & 0x00F;
  cmdbuf[7] = (zoom) & 0x00F;
  cmdbuf[8] = 0xFF;      
  return 0;
}

int setPanTilt(short pan, short tilt)
{
  pan = (int) (pan / 1571.0 * 790);
  tilt = (int) (tilt / 1571.0 * 790);
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x06;
  cmdbuf[3] = 0x02;
  cmdbuf[4] = 0x18;
  cmdbuf[5] = 0x14;

  cmdbuf[6] = (pan >> 12) & 0x00F;
  cmdbuf[7] = (pan >> 8) & 0x00F;
  cmdbuf[8] = (pan >> 4) & 0x00F;
  cmdbuf[9] = (pan) & 0x00F;

  cmdbuf[10] = (tilt >> 12) & 0x00F;
  cmdbuf[11] = (tilt >> 8) & 0x00F;
  cmdbuf[12] = (tilt >> 4) & 0x00F;
  cmdbuf[13] = (tilt) & 0x00F;

  cmdbuf[14] = 0xFF;
	       	       
  return 0;
}

int initPTZ ()
     /* Not used */ 
{
  // Interface clear command
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x00;
  cmdbuf[3] = 0x01;
  cmdbuf[4] = 0xFF;
  fprintf(stderr,"IF Clear");
  sleep(1);
  while(!readyPTZ());

  // Reset the PTZ
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x06;
  cmdbuf[3] = 0x05;
  cmdbuf[4] = 0xFF;
  
  fprintf(stderr,"Reset...wait");
  sleep(5);

  fprintf(stderr, "PTZPort initialized.\n");
  return 0;
}

int shutdownPTZ()
     /* Not used */
{
  fprintf(stderr,"PTZ Port Shutdown.\n");

  // Send a home message and stop it.
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x06;
  cmdbuf[3] = 0x04;
  cmdbuf[4] = 0xff;

  sleep(1);
  return 0;
}
  
int autoTrackSetup () {
  // AT_Mode On
  //  fprintf(stderr,"AT_On");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x01;
  cmdbuf[4] = 0x02;
  cmdbuf[5] = 0xFF;
  write(fd,cmdbuf,6); cmdbuf[0] = 0;
  usleep(100000);
    
  // Datascreen Off
  //  fprintf(stderr,"Datascreen Off");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x06;
  cmdbuf[3] = 0x06;
  cmdbuf[4] = 0x03;
  cmdbuf[5] = 0xFF;
  write(fd,cmdbuf,6); cmdbuf[0] = 0;
  usleep(100000);
   
  // AT_Entry1
  //  fprintf(stderr,"Entry1");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x15;
  cmdbuf[4] = 0x00;
  cmdbuf[5] = 0xFF;
  write(fd,cmdbuf,6); cmdbuf[0] = 0;
  usleep(100000);
  
  // AT_Chase2
  //  fprintf(stderr,"Chase2");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x07;
  cmdbuf[4] = 0x01;
  cmdbuf[5] = 0xFF;
  write(fd,cmdbuf,6); cmdbuf[0] = 0;
  usleep(100000);
  
  // AT/MD_Start/Stop
  /*
    fprintf(stderr,"AT_Start");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x06;
  cmdbuf[4] = 0x10;
  cmdbuf[5] = 0xFF;
  write(fd,cmdbuf,findCmdLength()); cmdbuf[0] = 0;
  usleep(100000);
  */
  
  // Frame_Display Off
  //  fprintf(stderr,"Frame_Display Off\n");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x04;
  cmdbuf[4] = 0x03;
  cmdbuf[5] = 0xFF;
  write(fd,cmdbuf,6); 
  cmdbuf[0] = 0;
  usleep(100000);

  return 0;
}

int autoTrackToggle() {
  // AT/MD_Start/Stop
  fprintf(stderr,"atToggle\n");
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x01;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x06;
  cmdbuf[4] = 0x10;
  cmdbuf[5] = 0xFF;
  while(!readyPTZ());

  return 0;
}

int getPTZ (int pt[3])
{
  // Pan-Tilt position query.
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x09;
  cmdbuf[2] = 0x06;
  cmdbuf[3] = 0x12;
  cmdbuf[4] = 0xFF;
  // Reply: Y0 50 0W 0W 0W 0W 0Z 0Z 0Z 0Z FF, WWWW: pan ZZZZ: tilt
  pan = 9999;
  tilt = 9999;
  while ((pan==9999)||(tilt==9999)); // wait for data
  pt[0] = (int) (pan / 790.0 * 1571);
  pt[1] = (int) (tilt / 790.0 * 1571);
  return 0;
}

int atModeInq (){
  // AT_Mode inq
  cmdbuf[0] = 0x81;
  cmdbuf[1] = 0x09;
  cmdbuf[2] = 0x07;
  cmdbuf[3] = 0x23;
  cmdbuf[4] = 0xFF;
  trackState = -1;
  while(trackState == -1);
  return trackState;
}


void *watchPTZ (void *arg)
{
  unsigned char msgbuf[128];
  unsigned char tempchar;
  int ii = 0;
  while (1) {
    usleep(100000);
    if (cmdbuf[0]!=0) {
      ii = 0;
      //      fprintf(stderr,"Sending ");
      // while (cmdbuf[ii]!=0xFF) fprintf(stderr," %02X",cmdbuf[ii++]);
      // fprintf(stderr,"\n");
      tempchar = cmdbuf[0];
      write(fd,cmdbuf,findCmdLength());
      cmdbuf[0] = 0; // wipe cmdbuf
      while (1) { 
 	for (ii=0;ii<128;ii++) msgbuf[ii] = 0; 
	read(fd,msgbuf,255);
	if (msgbuf[0] == 0x90) {
	  ii = 0;
	  
	  // fprintf(stderr,"Got ");
	  // while ((msgbuf[ii]!=0xFF) && (ii<14)) fprintf(stderr," %02X",msgbuf[ii++]);
	  // fprintf(stderr," Size: %d",ii);
	  // fprintf(stderr,"\n");
	  while ((msgbuf[ii++]!=0xFF) && (ii<14));
	  if (ii>13) { // ERROR: Command sequences are NEVER more than 14
	    //   fprintf(stderr,"Bad error!!\n");
	    cmdbuf[0] = tempchar;
	    break;
	  }  
	  
	  if (msgbuf[2] == 0xFF) {
	    if (msgbuf[1] == 0x41) ;//fprintf(stderr,"ACK1\n");
	    if (msgbuf[1] == 0x42) ;//fprintf(stderr,"ACK2\n");
	    if (msgbuf[1] == 0x51) {
	      // fprintf(stderr,"CmdComplete1\n"); 
	      break;}
	    if (msgbuf[1] == 0x52) {
	      //fprintf(stderr,"CmdComplete2\n"); 
	      break;}
	  }
	  if (msgbuf[3] == 0xFF) {
	    if (msgbuf[1] == 0x60) {
	      fprintf(stderr,"GenericError.\n"); 
	      break;}
	  }
	  if (msgbuf[4] == 0xFF) {
	    if (msgbuf[1] == 0x50) {
	      trackState = 0x00F & msgbuf[2];
	      // fprintf(stderr,"trackState:%d\n",trackState);
	      break;}
	  }
	  if (msgbuf[10] == 0xFF) {
	    if (msgbuf[1] == 0x50) {
	      pan = ((msgbuf[2]<<12)&0xF000) + 
		((msgbuf[3]<<8)&0x0F00) + 
		((msgbuf[4]<<4)&0x00F0) +
		((msgbuf[5])&0x000F);
	      tilt = ((msgbuf[6]<<12)&0xF000) + 
		((msgbuf[7]<<8)&0x0F00) + 
		((msgbuf[8]<<4)&0x00F0) +
		((msgbuf[9])&0x000F);
	      break;
	    }
	  }
	}
	usleep(10000);
      }
    }
  }
}

int startPTZ()
{
  pthread_t *watcher;

  cmdbuf[0] = 0;
  
  fd = openSerialPort("/dev/ttyS3", 9600);
  
  watcher = malloc(sizeof(pthread_t)); 
  pthread_create(watcher,NULL,&watchPTZ,NULL);
  
  while (!readyPTZ());
    
  return 0;
}













